﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using System.Data;

using Npgsql;

using Dapper;

using TestServer.Models;

namespace TestServer.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class DataController : ControllerBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region INQUIRY_LIST_SQL_POSTGRESQL

        /// <summary>
        /// INQUIRY_LIST_SQL (POSTGRESQL용)
        /// </summary>
        private static string INQUIRY_LIST_SQL_POSTGRESQL = @"
SELECT
    A.ID           AS ID
   ,A.MENU_ID      AS MENU_ID
   ,A.SUBJECT      AS SUBJECT
   ,A.WRITE_DATE   AS WRITE_DATE
   ,A.SEQUENCE     AS SEQUENCE
   ,A.MEMBER_GRADE AS MEMBER_GRADE
   ,A.LIMIT_AGE    AS LIMIT_AGE
   ,A.DATA_TYPE    AS DATA_TYPE
   ,A.CATEGORY1    AS CATEGORY1
   ,A.CATEGORY2    AS CATEGORY2
   ,A.CATEGORY3    AS CATEGORY3
   ,A.CATEGORY4    AS CATEGORY4
   ,A.CATEGORY5    AS CATEGORY5
   ,A.CATEGORY6    AS CATEGORY6
   ,A.CATEGORY7    AS CATEGORY7
   ,A.CATEGORY8    AS CATEGORY8
   ,A.STORAGE_ID   AS STORAGE_ID
   ,B.NAME         AS STORAGE_NAME
   ,A.CREATE_ID    AS CREATE_ID
   ,C.USER_ID      AS CREATE_USER_ID
   ,A.CREATE_TIME  AS CREATE_TIME
   ,A.UPDATE_ID    AS UPDATE_ID
   ,D.USER_ID      AS UPDATE_USER_ID
   ,A.UPDATE_TIME  AS UPDATE_TIME
FROM      LIBRARY_DATA A
LEFT JOIN COM_STORAGE  B ON B.ID = A.STORAGE_ID
LEFT JOIN COM_USER     C ON C.ID = A.CREATE_ID
LEFT JOIN COM_USER     D ON D.ID = A.UPDATE_ID
WHERE A.MENU_ID = @MENU_ID
AND   (@ID           =  '' OR @ID           IS NULL OR A.ID        ILIKE @ID           || '%')
AND   (@SUBJECT      =  '' OR @SUBJECT      IS NULL OR A.SUBJECT   ILIKE @SUBJECT      || '%')
AND   (@SEQUENCE     =  '' OR @SEQUENCE     IS NULL OR A.SEQUENCE  ILIKE @SEQUENCE     || '%')
AND   A.MEMBER_GRADE <= @MEMBER_GRADE
AND   A.LIMIT_AGE    <= @LIMIT_AGE
AND   A.WRITE_DATE BETWEEN @WRITE_DATE1 AND @WRITE_DATE2
AND   (@DATA_TYPE    =  '' OR @DATA_TYPE    IS NULL OR A.DATA_TYPE =     @DATA_TYPE          )
AND   (@CATEGORY1    =  '' OR @CATEGORY1    IS NULL OR A.CATEGORY1 ILIKE @CATEGORY1    || '%')
AND   (@CATEGORY2    =  '' OR @CATEGORY2    IS NULL OR A.CATEGORY2 ILIKE @CATEGORY2    || '%')
AND   (@CATEGORY3    =  '' OR @CATEGORY3    IS NULL OR A.CATEGORY3 ILIKE @CATEGORY3    || '%')
AND   (@CATEGORY4    =  '' OR @CATEGORY4    IS NULL OR A.CATEGORY4 ILIKE @CATEGORY4    || '%')
AND   (@CATEGORY5    =  '' OR @CATEGORY5    IS NULL OR A.CATEGORY5 ILIKE @CATEGORY5    || '%')
AND   (@CATEGORY6    =  '' OR @CATEGORY6    IS NULL OR A.CATEGORY6 ILIKE @CATEGORY6    || '%')
AND   (@CATEGORY7    =  '' OR @CATEGORY7    IS NULL OR A.CATEGORY7 ILIKE @CATEGORY7    || '%')
AND   (@CATEGORY8    =  '' OR @CATEGORY8    IS NULL OR A.CATEGORY8 ILIKE @CATEGORY8    || '%')
AND   (@STORAGE_NAME =  '' OR @STORAGE_NAME IS NULL OR B.NAME      ILIKE @STORAGE_NAME || '%')
AND   A.CREATE_TIME BETWEEN @CREATE_TIME1 AND @CREATE_TIME2
ORDER BY
    A.SEQUENCE    DESC
   ,A.WRITE_DATE  DESC
   ,A.CREATE_TIME DESC
   ,A.SUBJECT     ASC
";

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 구성
        /// </summary>
        private readonly IConfiguration configuration;

        /// <summary>
        /// 로거
        /// </summary>
        private readonly ILogger<DataController> logger;

        /// <summary>
        /// 연결 문자열
        /// </summary>
        private readonly string connectionString;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DataController(configuration, logger)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="configuration">구성</param>
        /// <param name="logger">로거</param>
        public DataController(IConfiguration configuration, ILogger<DataController> logger)
        {
            this.configuration    = configuration;
            this.logger           = logger;
            this.connectionString = configuration.GetConnectionString("DefaultConnection");
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리스트 조회하기 - InquiryList(searchCondition, transaction)

        /// <summary>
        /// 리스트 조회하기
        /// </summary>
        /// <param name="searchCondition">조회 조건</param>
        /// <param name="transaction">트랜잭션</param>
        /// <returns>조회 결과</returns>
        [HttpPost]
        [Route("InquiryList")]
        public IEnumerable<DataModel> InquiryList([FromBody] DataSearchConditionModel searchCondition)
        {
            using(IDbConnection connection = CreateConnection())
            {
                using(IDataReader reader = connection.ExecuteReader(INQUIRY_LIST_SQL_POSTGRESQL, searchCondition))
                {
                    while(reader.Read())
                    {
                        var rowParser = reader.GetRowParser<DataModel>(typeof(DataModel));

                        yield return rowParser(reader);
                    }
                }
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 연결 생성하기 - CreateConnection()

        /// <summary>
        /// 연결 생성하기
        /// </summary>
        /// <returns>연결</returns>
        private IDbConnection CreateConnection()
        {
            return new NpgsqlConnection(this.connectionString);
        }

        #endregion
    }
}